#include "NeuralNetwork.hpp"

torch::Device NeuralNetworkImpl::device(torch::cuda::is_available() ? torch::kCUDA : torch::kCPU);

float NeuralNetworkImpl::entropy(torch::Tensor probabilities)
{
  if (probabilities.dim() != 1)
    util::myThrow("Invalid probabilities tensor");

  probabilities = probabilities.unsqueeze(0);
  auto logProbs = torch::clamp(torch::log(torch::transpose(probabilities, 0, 1)), -10.0, 10.0);
  logProbs.index({torch::isnan(logProbs)}) = 0.0;
  return - torch::tensordot(probabilities, logProbs, {0,1}, {0,1}).item<float>();
}

